@@ -9,9 +9,10 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import unittest
-from xml.etree.ElementTree import Element, parse as etree_parse
+import warnings
 from pathlib import Path
 from typing import cast, MutableMapping, Optional, Type
+from xml.etree.ElementTree import Element, parse as etree_parse
 
 try:
     import lxml.etree as lxml_etree
@@ -22,11 +23,11 @@ from elementpath.etree import etree_tostring
 
 from xmlschema import XMLSchema, XMLSchemaValidationError, fetch_namespaces
 from xmlschema.dataobjects import DataElement
-from xmlschema.testing import etree_elements_assert_equal
+from xmlschema.testing import etree_elements_assert_equal, run_xmlschema_tests
 
 from xmlschema.converters import XMLSchemaConverter, UnorderedConverter, \
     ParkerConverter, BadgerFishConverter, AbderaConverter, JsonMLConverter, \
-    ColumnarConverter
+    ColumnarConverter, GDataConverter
 from xmlschema.dataobjects import DataElementConverter
 
 
@@ -53,6 +54,9 @@ class TestConverters(unittest.TestCase):
         else:
             cls.col_lxml_root = None
 
+        cls.vh_xsd_filename = cls.casepath('examples/vehicles/vehicles.xsd')
+        cls.vh_xml_filename = cls.casepath('examples/vehicles/vehicles.xml')
+
     @classmethod
     def casepath(cls, relative_path):
         return str(Path(__file__).parent.joinpath('test_cases', relative_path))
@@ -61,14 +65,17 @@ class TestConverters(unittest.TestCase):
         converter = XMLSchemaConverter()
         self.assertIs(converter.etree_element_class, Element)
 
-        converter = XMLSchemaConverter(etree_element_class=Element)
-        self.assertIs(converter.etree_element_class, Element)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
 
-        if lxml_etree is not None:
-            converter = XMLSchemaConverter(
-                etree_element_class=cast(Type[Element], lxml_etree.Element)
-            )
-            self.assertIs(converter.etree_element_class, lxml_etree.Element)
+            converter = XMLSchemaConverter(etree_element_class=Element)
+            self.assertIs(converter.etree_element_class, Element)
+
+            if lxml_etree is not None:
+                converter = XMLSchemaConverter(
+                    etree_element_class=cast(Type[Element], lxml_etree.Element)
+                )
+                self.assertIs(converter.etree_element_class, lxml_etree.Element)
 
     def test_prefix_arguments(self):
         converter = XMLSchemaConverter(cdata_prefix='#')
@@ -96,6 +103,24 @@ class TestConverters(unittest.TestCase):
         self.assertIn('@xmlns:', str(col_schema.decode(col_xml_filename, strip_namespaces=False)))
         self.assertNotIn('@xmlns:', str(col_schema.decode(col_xml_filename)))
 
+    def test_arguments_with_wrong_types(self):
+
+        with self.assertRaises(TypeError) as ctx:
+            XMLSchemaConverter(cdata_prefix=1)
+        self.assertTrue(str(ctx.exception).startswith("invalid type <class 'int'>"))
+
+        with self.assertRaises(TypeError) as ctx:
+            XMLSchemaConverter(preserve_root=1)
+        self.assertTrue(str(ctx.exception).startswith("invalid type <class 'int'>"))
+
+        with self.assertRaises(TypeError) as ctx:
+            XMLSchemaConverter(dict_class=list)
+        self.assertTrue(str(ctx.exception).startswith("invalid <class 'list'>"))
+
+        with self.assertRaises(TypeError) as ctx:
+            XMLSchemaConverter(list_class=dict)
+        self.assertTrue(str(ctx.exception).startswith("invalid <class 'dict'>"))
+
     def test_lossy_property(self):
         self.assertTrue(XMLSchemaConverter().lossy)
         self.assertFalse(XMLSchemaConverter(cdata_prefix='#').lossy)
@@ -157,11 +182,15 @@ class TestConverters(unittest.TestCase):
 
     def test_etree_element_method(self):
         converter = XMLSchemaConverter()
-        elem = converter.etree_element('A')
-        self.assertIsNone(etree_elements_assert_equal(elem, Element('A')))
 
-        elem = converter.etree_element('A', attrib={})
-        self.assertIsNone(etree_elements_assert_equal(elem, Element('A')))
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+
+            elem = converter.etree_element('A')
+            self.assertIsNone(etree_elements_assert_equal(elem, Element('A')))
+
+            elem = converter.etree_element('A', attrib={})
+            self.assertIsNone(etree_elements_assert_equal(elem, Element('A')))
 
     def test_columnar_converter(self):
         col_schema = XMLSchema(self.col_xsd_filename, converter=ColumnarConverter)
@@ -273,6 +302,24 @@ class TestConverters(unittest.TestCase):
         root = col_schema.encode(obj2, preserve_root=True)  # No namespace unmap is required
         self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
 
+    @unittest.skipIf(lxml_etree is None, 'lxml is not available')
+    def test_decode_encode_default_converter_with_lxml(self):
+        vh_schema = XMLSchema(self.vh_xsd_filename)
+        vh_lxml_root = lxml_etree.parse(self.vh_xml_filename).getroot()
+        etree_element_class = cast(Type[Element], lxml_etree.Element)
+
+        # Decode from XML file
+        obj1 = vh_schema.decode(vh_lxml_root, process_namespaces=False)
+        self.assertNotIn("'@xmlns:vh'", repr(obj1))
+
+        obj1 = vh_schema.decode(vh_lxml_root)
+        self.assertIn("'@xmlns:vh'", repr(obj1))
+
+        root = vh_schema.encode(obj1, etree_element_class=etree_element_class)
+        self.assertIsNone(
+            etree_elements_assert_equal(vh_lxml_root, root, strict=False, check_nsmap=True)
+        )
+
     def test_decode_encode_unordered_converter(self):
         col_schema = XMLSchema(self.col_xsd_filename, converter=UnorderedConverter)
 
@@ -386,6 +433,34 @@ class TestConverters(unittest.TestCase):
         root = col_schema.encode(obj2)  # No namespace unmap is required
         self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
 
+    def test_decode_encode_gdata_converter(self):
+        col_schema = XMLSchema(self.col_xsd_filename, converter=GDataConverter)
+
+        obj1 = col_schema.decode(self.col_xml_filename)
+        self.assertIn("'xmlns$col'", repr(obj1))
+
+        root = col_schema.encode(obj1, path='./col:collection', namespaces=self.col_nsmap)
+        self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
+
+        root = col_schema.encode(obj1)
+        self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
+
+        # With ElementTree namespaces are not mapped
+        obj2 = col_schema.decode(self.col_xml_root)
+        self.assertNotIn("'@xmlns'", repr(obj2))
+        self.assertNotEqual(obj1, obj2)
+        self.assertEqual(obj1, col_schema.decode(self.col_xml_root, namespaces=self.col_nsmap))
+
+        # With lxml.etree namespaces are mapped
+        if self.col_lxml_root is not None:
+            self.assertEqual(obj1, col_schema.decode(self.col_lxml_root))
+
+        root = col_schema.encode(obj2, path='./col:collection', namespaces=self.col_nsmap)
+        self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
+
+        root = col_schema.encode(obj2)  # No namespace unmap is required
+        self.assertIsNone(etree_elements_assert_equal(self.col_xml_root, root, strict=False))
+
     def test_decode_encode_abdera_converter(self):
         col_schema = XMLSchema(self.col_xsd_filename, converter=AbderaConverter)
 
@@ -520,10 +595,14 @@ class TestConverters(unittest.TestCase):
 
         default_xml_filename = self.casepath('examples/collection/collection-default.xml')
         obj1 = col_schema.decode(default_xml_filename)
-        self.assertIn('@xmlns', obj1)
+        assert isinstance(obj1, dict)
+        self.assertIn('@xmlns', obj1['collection'])
         self.assertEqual(repr(obj1).count("'@xmlns'"), 1)
-        self.assertEqual(obj1['@xmlns'], {'$': 'http://example.com/ns/collection',
-                                          'xsi': 'http://www.w3.org/2001/XMLSchema-instance'})
+        self.assertEqual(
+            obj1['collection']['@xmlns'],
+            {'$': 'http://example.com/ns/collection',
+             'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}
+        )
 
         root = col_schema.encode(obj1)
         default_xml_root = etree_parse(default_xml_filename).getroot()
@@ -604,20 +683,17 @@ class TestConverters(unittest.TestCase):
                 continue
 
             if k == 4:
-                self.assertEqual(obj, {'@xmlns': {'tst': 'http://xmlschema.test/ns'},
-                                       'tst:e1': {'@a1': 'foo', 'e2': [{}, {}], '$1': 'bar'}})
+                self.assertEqual(obj, {'tst:e1': {'@a1': 'foo',
+                                                  '@xmlns': {'tst': 'http://xmlschema.test/ns'},
+                                                  'e2': [{}, {}], '$1': 'bar'}})
             else:
-                self.assertEqual(obj, {'@xmlns': {'tst': 'http://xmlschema.test/ns'},
-                                       'tst:e1': {'@a1': 'foo', 'e2': [{}], '$1': 'bar'}})
+                self.assertEqual(obj, {'tst:e1': {'@a1': 'foo',
+                                                  '@xmlns': {'tst': 'http://xmlschema.test/ns'},
+                                                  'e2': [{}], '$1': 'bar'}})
 
             text = etree_tostring(root, namespaces={'tst': 'http://xmlschema.test/ns'})
             self.assertEqual(len(text.split('bar')), 2)
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema converters with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    run_xmlschema_tests('converters')
