@@ -9,6 +9,7 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import unittest
+import copy
 import xml.etree.ElementTree as ElementTree
 from pathlib import Path
 from typing import Dict
@@ -18,10 +19,10 @@ from xmlschema import XMLSchema10, XMLSchema11, fetch_namespaces, etree_tostring
     XsdElement, XsdAttribute, XsdType
 
 from xmlschema.validators import XsdAttributeGroup
-from xmlschema.helpers import is_etree_element
+from xmlschema.utils.etree import is_etree_element
 from xmlschema.names import XSI_TYPE
 from xmlschema.dataobjects import DataBindingMeta, DataBindingConverter
-from xmlschema.testing import etree_elements_assert_equal
+from xmlschema.testing import etree_elements_assert_equal, run_xmlschema_tests
 
 
 class TestDataElementInterface(unittest.TestCase):
@@ -223,6 +224,52 @@ class TestDataObjects(unittest.TestCase):
             col_data.xsd_type = col_data[0].xsd_type
         self.assertEqual(str(ec.exception), "the instance is already bound to another XSD type")
 
+    def test_xmlns_processing_argument(self):
+        xsd_file = self.casepath('examples/collection/collection5.xsd')
+        xml_file = self.casepath('examples/collection/collection-redef-xmlns.xml')
+
+        xmlns0 = [('col1', 'http://example.com/ns/collection'),
+                  ('col', 'http://xmlschema.test/ns'),
+                  ('', 'http://xmlschema.test/ns'),
+                  ('xsi', 'http://www.w3.org/2001/XMLSchema-instance')]
+        nsmap0 = dict(xmlns0)
+        xmlns1 = [('col', 'http://example.com/ns/collection')]
+        nsmap1 = dict(xmlns0)
+        nsmap1.update(xmlns1)
+        schema = self.schema_class(xsd_file)
+        self.assertTrue(schema.is_valid(xml_file))
+
+        resource = XMLResource(xml_file)
+        obj = schema.decode(resource, converter=self.converter)
+        self.assertListEqual(obj.xmlns, xmlns0)
+        self.assertDictEqual(obj.nsmap, nsmap0)
+        self.assertListEqual(obj[0].xmlns, xmlns1)
+        self.assertDictEqual(obj[0].nsmap, nsmap1)
+        root = schema.encode(obj, converter=self.converter)
+        self.assertTrue(is_etree_element(root))
+
+        obj = schema.decode(resource, converter=self.converter, xmlns_processing='stacked')
+        self.assertListEqual(obj.xmlns, xmlns0)
+        self.assertDictEqual(obj.nsmap, nsmap0)
+        self.assertListEqual(obj[0].xmlns, xmlns1)
+        self.assertDictEqual(obj[0].nsmap, nsmap1)
+        root = schema.encode(obj, converter=self.converter, xmlns_processing='stacked')
+        self.assertTrue(is_etree_element(root))
+
+        obj = schema.decode(resource, converter=self.converter, xmlns_processing='collapsed')
+        root = schema.encode(obj, converter=self.converter, xmlns_processing='collapsed')
+        self.assertTrue(is_etree_element(root))
+
+        obj = schema.decode(resource, converter=self.converter, xmlns_processing='root-only')
+        root = schema.encode(obj, converter=self.converter, xmlns_processing='root-only')
+        self.assertTrue(is_etree_element(root))
+
+        obj = schema.decode(resource, converter=self.converter, xmlns_processing='none')
+        root = schema.encode(obj, converter=self.converter, xmlns_processing='none')
+        self.assertTrue(is_etree_element(root))
+        for data_element in obj.iter():
+            self.assertDictEqual(data_element.nsmap, {})
+
     def test_encode_to_element_tree(self):
         col_data = self.col_schema.decode(self.col_xml_filename)
 
@@ -368,15 +415,15 @@ class TestDataObjects(unittest.TestCase):
         col_data = self.col_schema.decode(self.col_xml_filename)
         self.assertTrue(col_data.is_valid())
 
-        col_data[0][0].value = '1'
+        col_data[0][0].value = '1.0'
         with self.assertRaises(XMLSchemaValidationError) as ec:
             col_data.validate()
-        self.assertIn("'1' is not an instance of <class 'int'>", str(ec.exception))
+        self.assertIn("invalid literal for int() with base 10", str(ec.exception))
         self.assertFalse(col_data.is_valid())
 
         errors = list(col_data.iter_errors())
         self.assertEqual(len(errors), 1)
-        self.assertIn("'1' is not an instance of <class 'int'>", str(errors[0]))
+        self.assertIn("invalid literal for int() with base 10", str(errors[0]))
 
         col_data.find('object/position').value = 1
         self.assertTrue(col_data.is_valid())
@@ -428,7 +475,7 @@ class TestDataObjects(unittest.TestCase):
 
         converter = self.converter(data_element_class=MyDataElement)
         self.assertIs(converter.data_element_class, MyDataElement)
-        self.assertIs(converter.copy().data_element_class, MyDataElement)
+        self.assertIs(copy.copy(converter).data_element_class, MyDataElement)
 
         self.col_schema.maps.clear_bindings()  # needed for DataBindingConverter ...
         col_data = self.col_schema.decode(self.col_xml_filename, converter=converter)
@@ -605,9 +652,4 @@ class TestDataBindings11(TestDataBindings):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema data objects with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    run_xmlschema_tests('data objects')
