@@ -13,6 +13,7 @@ import os
 import base64
 import json
 import math
+import pathlib
 from decimal import Decimal
 from collections.abc import MutableMapping, MutableSequence, Set
 from textwrap import dedent
@@ -61,6 +62,58 @@ VEHICLES_DICT_ALT = [
     {'@xsi:schemaLocation': 'http://example.com/vehicles vehicles.xsd'}
 ]
 
+VEHICLES_DICT_OVERRIDE_PREFIX_1 = {
+    '@xmlns:vh': 'http://example.com/vehicles',
+    '@xmlns:vh2': 'http://example.com/vehicles',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/vehicles vehicles.xsd',
+    'vh2:cars': {
+        'vh2:car': [
+            {'@make': 'Porsche', '@model': '911'},
+            {'@make': 'Porsche', '@model': '911'}
+        ]},
+    'vh2:bikes': {
+        'vh2:bike': [
+            {'@make': 'Harley-Davidson', '@model': 'WL'},
+            {'@make': 'Yamaha', '@model': 'XS650'}
+        ]}
+}
+
+VEHICLES_DICT_OVERRIDE_PREFIX_2 = {
+    '@xmlns': 'http://example.com/vehicles',
+    '@xmlns:vh': 'http://example.com/vehicles',
+    '@xmlns:vh-alt': 'http://example.com/vehicles',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/vehicles vehicles.xsd',
+    'vh-alt:cars': {
+        'vh-alt:car': [
+            {'@make': 'Porsche', '@model': '911'},
+            {'@make': 'Porsche', '@model': '911'}
+        ]},
+    'vh-alt:bikes': {
+        'vh-alt:bike': [
+            {'@make': 'Harley-Davidson', '@model': 'WL'},
+            {'@make': 'Yamaha', '@model': 'XS650'}
+        ]}
+}
+
+VEHICLES_DICT_OVERRIDE_PREFIX_3 = {
+    '@xmlns': 'http://example.com/vehicles',
+    '@xmlns:vh': 'http://example.com/vehicles',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/vehicles vehicles.xsd',
+    'cars': {
+        'car': [
+            {'@make': 'Porsche', '@model': '911'},
+            {'@make': 'Porsche', '@model': '911'}
+        ]},
+    'bikes': {
+        'bike': [
+            {'@make': 'Harley-Davidson', '@model': 'WL'},
+            {'@make': 'Yamaha', '@model': 'XS650'}
+        ]}
+}
+
 COLLECTION_DICT = {
     '@xmlns:col': 'http://example.com/ns/collection',
     '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
@@ -86,7 +139,7 @@ COLLECTION_DICT = {
                 '@id': 'JM',
                 'born': '1893-04-20',
                 'dead': '1983-12-25',
-                'name': u'Joan Miró',
+                'name': 'Joan Miró',
                 'qualification': 'painter, sculptor and ceramicist'
             },
             'position': 2,
@@ -95,6 +148,15 @@ COLLECTION_DICT = {
     }]
 }
 
+MENU_DICT = {
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:noNamespaceSchemaLocation': 'menù.xsd',
+    'antipasto': ['Affettati misti', 'Bruschetta', 'Polenta e funghi'],
+    'primo': ['Lasagne', 'Gnocchi al ragù', 'Risotto allo zafferano'],
+    'secondo': ['Tagliata di pollo', 'Cotoletta alla milanese', 'Caprese'],
+    'dolce': ['Crostata ai mirtilli', 'Tiramisù']
+}
+
 COLLECTION_PARKER = {
     'object': [{'author': {'born': '1841-02-25',
                            'dead': '1919-12-03',
@@ -106,7 +168,7 @@ COLLECTION_PARKER = {
                 'year': '1886'},
                {'author': {'born': '1893-04-20',
                            'dead': '1983-12-25',
-                           'name': u'Joan Miró',
+                           'name': 'Joan Miró',
                            'qualification': 'painter, sculptor and ceramicist'},
                 'position': 2,
                 'title': None,
@@ -123,18 +185,18 @@ COLLECTION_PARKER_ROOT = {
                                    'year': '1886'},
                                   {'author': {'born': '1893-04-20',
                                               'dead': '1983-12-25',
-                                              'name': u'Joan Miró',
+                                              'name': 'Joan Miró',
                                               'qualification': 'painter, sculptor and ceramicist'},
                                    'position': 2,
                                    'title': None,
                                    'year': '1925'}]}}
 
 COLLECTION_BADGERFISH = {
-    '@xmlns': {
-        'col': 'http://example.com/ns/collection',
-        'xsi': 'http://www.w3.org/2001/XMLSchema-instance'},
     'col:collection': {
         '@xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd',
+        '@xmlns': {
+            'col': 'http://example.com/ns/collection',
+            'xsi': 'http://www.w3.org/2001/XMLSchema-instance'},
         'object': [{
             '@available': True,
             '@id': 'b0836217462',
@@ -155,7 +217,7 @@ COLLECTION_BADGERFISH = {
                     '@id': 'JM',
                     'born': {'$': '1893-04-20'},
                     'dead': {'$': '1983-12-25'},
-                    'name': {'$': u'Joan Miró'},
+                    'name': {'$': 'Joan Miró'},
                     'qualification': {
                         '$': 'painter, sculptor and ceramicist'}
                 },
@@ -167,46 +229,48 @@ COLLECTION_BADGERFISH = {
 }
 
 COLLECTION_ABDERA = {
-    'attributes': {
-        'xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd'
-    },
-    'children': [
-        {
-            'object': [
-                {
-                    'attributes': {'available': True, 'id': 'b0836217462'},
-                    'children': [{
-                        'author': {
-                            'attributes': {'id': 'PAR'},
-                            'children': [{
-                                'born': '1841-02-25',
-                                'dead': '1919-12-03',
-                                'name': 'Pierre-Auguste Renoir',
-                                'qualification': 'painter'}
-                            ]},
-                        'estimation': 10000.0,
-                        'position': 1,
-                        'title': 'The Umbrellas',
-                        'year': '1886'}
-                    ]},
-                {
-                    'attributes': {'available': True, 'id': 'b0836217463'},
-                    'children': [{
-                        'author': {
-                            'attributes': {'id': 'JM'},
-                            'children': [{
-                                'born': '1893-04-20',
-                                'dead': '1983-12-25',
-                                'name': u'Joan Miró',
-                                'qualification': 'painter, sculptor and ceramicist'}
-                            ]},
-                        'position': 2,
-                        'title': [],
-                        'year': '1925'
+    'col:collection': {
+        'attributes': {
+            'xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd'
+        },
+        'children': [
+            {
+                'object': [
+                    {
+                        'attributes': {'available': True, 'id': 'b0836217462'},
+                        'children': [{
+                            'author': {
+                                'attributes': {'id': 'PAR'},
+                                'children': [{
+                                    'born': '1841-02-25',
+                                    'dead': '1919-12-03',
+                                    'name': 'Pierre-Auguste Renoir',
+                                    'qualification': 'painter'}
+                                ]},
+                            'estimation': 10000.0,
+                            'position': 1,
+                            'title': 'The Umbrellas',
+                            'year': '1886'}
+                        ]},
+                    {
+                        'attributes': {'available': True, 'id': 'b0836217463'},
+                        'children': [{
+                            'author': {
+                                'attributes': {'id': 'JM'},
+                                'children': [{
+                                    'born': '1893-04-20',
+                                    'dead': '1983-12-25',
+                                    'name': 'Joan Miró',
+                                    'qualification': 'painter, sculptor and ceramicist'}
+                                ]},
+                            'position': 2,
+                            'title': [],
+                            'year': '1925'
+                        }]
                     }]
-                }]
-        }
-    ]}
+            }
+        ]}
+}
 
 COLLECTION_JSON_ML = [
     'col:collection',
@@ -238,14 +302,13 @@ COLLECTION_JSON_ML = [
      [
          'author',
          {'id': 'JM'},
-         ['name', u'Joan Miró'],
+         ['name', 'Joan Miró'],
          ['born', '1893-04-20'],
          ['dead', '1983-12-25'],
          ['qualification', 'painter, sculptor and ceramicist']
      ]]
 ]
 
-
 COLLECTION_COLUMNAR = {
     'collection': {
         'collectionxsi:schemaLocation': 'http://example.com/ns/collection collection.xsd',
@@ -312,6 +375,150 @@ COLLECTION_COLUMNAR_ = {
     }
 }
 
+COLLECTION_XMLNS_PROCESSING_STACKED = {
+    '@xmlns:col1': 'http://example.com/ns/collection',
+    '@xmlns:col': 'http://xmlschema.test/ns',
+    '@xmlns': 'http://xmlschema.test/ns',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/ns/collection collection5.xsd',
+    'col:object': [
+        {'@xmlns:col': 'http://example.com/ns/collection',
+         '@id': 'b0836217462',
+         '@available': True,
+         'col:position': 1,
+         'col:title': 'The Umbrellas',
+         'col:year': '1886',
+         'col:author': {
+             '@id': 'PAR',
+             'col:name': 'Pierre-Auguste Renoir',
+             'col:born': '1841-02-25',
+             'col:dead': '1919-12-03',
+             'col:qualification': 'painter'
+         },
+         'col:estimation': Decimal('10000.00')}],
+    'object': [
+        {'@xmlns': 'http://example.com/ns/collection',
+         '@id': 'b0836217463',
+         '@available': True,
+         'position': 2,
+         'title': None,
+         'year': '1925',
+         'author': {
+            '@id': 'JM',
+            'name': 'Joan Miró',
+            'born': '1893-04-20',
+            'dead': '1983-12-25',
+            'qualification': 'painter, sculptor and ceramicist'
+         }}
+    ]
+}
+
+COLLECTION_XMLNS_PROCESSING_COLLAPSED = {
+    '@xmlns': 'http://xmlschema.test/ns',
+    '@xmlns:col': 'http://xmlschema.test/ns',
+    '@xmlns:col0': 'http://example.com/ns/collection',
+    '@xmlns:col1': 'http://example.com/ns/collection',
+    '@xmlns:default': 'http://example.com/ns/collection',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/ns/collection collection5.xsd',
+    'col1:object': [
+        {'@id': 'b0836217462',
+         '@available': True,
+         'col1:position': 1,
+         'col1:title': 'The Umbrellas',
+         'col1:year': '1886',
+         'col1:author': {
+             '@id': 'PAR',
+             'col1:name': 'Pierre-Auguste Renoir',
+             'col1:born': '1841-02-25',
+             'col1:dead': '1919-12-03',
+             'col1:qualification': 'painter'
+         },
+         'col1:estimation': Decimal('10000.00')},
+        {'@id': 'b0836217463',
+         '@available': True,
+         'col1:position': 2,
+         'col1:title': None,
+         'col1:year': '1925',
+         'col1:author': {
+            '@id': 'JM',
+            'col1:name': 'Joan Miró',
+            'col1:born': '1893-04-20',
+            'col1:dead': '1983-12-25',
+            'col1:qualification': 'painter, sculptor and ceramicist'
+         }}
+    ]
+}
+
+COLLECTION_XMLNS_PROCESSING_ROOT_ONLY = {
+    '@xmlns': 'http://xmlschema.test/ns',
+    '@xmlns:col': 'http://xmlschema.test/ns',
+    '@xmlns:col1': 'http://example.com/ns/collection',
+    '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi:schemaLocation': 'http://example.com/ns/collection collection5.xsd',
+    'col1:object': [
+        {'@id': 'b0836217462',
+         '@available': True,
+         'col1:position': 1,
+         'col1:title': 'The Umbrellas',
+         'col1:year': '1886',
+         'col1:author': {
+             '@id': 'PAR',
+             'col1:name': 'Pierre-Auguste Renoir',
+             'col1:born': '1841-02-25',
+             'col1:dead': '1919-12-03',
+             'col1:qualification': 'painter'
+         },
+         'col1:estimation': Decimal('10000.00')},
+        {'@id': 'b0836217463',
+         '@available': True,
+         'col1:position': 2,
+         'col1:title': None,
+         'col1:year': '1925',
+         'col1:author': {
+            '@id': 'JM',
+            'col1:name': 'Joan Miró',
+            'col1:born': '1893-04-20',
+            'col1:dead': '1983-12-25',
+            'col1:qualification': 'painter, sculptor and ceramicist'
+         }}
+    ]
+}
+
+
+COLLECTION_XMLNS_PROCESSING_NONE = {
+    '@xmlns:col-alt': 'http://example.com/ns/collection',
+    '@xmlns:xsi-alt': 'http://www.w3.org/2001/XMLSchema-instance',
+    '@xsi-alt:schemaLocation': 'http://example.com/ns/collection collection5.xsd',
+    'col-alt:object': [
+        {'@id': 'b0836217462',
+         '@available': True,
+         'col-alt:position': 1,
+         'col-alt:title': 'The Umbrellas',
+         'col-alt:year': '1886',
+         'col-alt:author': {
+             '@id': 'PAR',
+             'col-alt:name': 'Pierre-Auguste Renoir',
+             'col-alt:born': '1841-02-25',
+             'col-alt:dead': '1919-12-03',
+             'col-alt:qualification': 'painter'
+         },
+         'col-alt:estimation': Decimal('10000.00')},
+        {'@id': 'b0836217463',
+         '@available': True,
+         'col-alt:position': 2,
+         'col-alt:title': None,
+         'col-alt:year': '1925',
+         'col-alt:author': {
+            '@id': 'JM',
+            'col-alt:name': 'Joan Miró',
+            'col-alt:born': '1893-04-20',
+            'col-alt:dead': '1983-12-25',
+            'col-alt:qualification': 'painter, sculptor and ceramicist'
+         }}
+    ]
+}
+
 DATA_DICT = {
     '@xmlns:ns': 'ns',
     '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
@@ -323,6 +530,7 @@ DATA_DICT = {
     'decimal_value': [Decimal('1')],
     'hexbin': 'AABBCCDD',
     'menù': 'baccalà mantecato',
+    'qname': 'ns:foo',
     'complex_boolean': [
         {'$': True, '@Type': 2}, {'$': False, '@Type': 1}, True, False
     ],
@@ -344,7 +552,8 @@ def iter_nested_iterables(obj):
 
 
 class TestDecoding(XsdValidatorTestCase):
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
+
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
 
     def check_decode(self, xsd_component, data, expected, **kwargs):
         if isinstance(expected, type) and issubclass(expected, Exception):
@@ -473,7 +682,8 @@ class TestDecoding(XsdValidatorTestCase):
             </xs:schema>""")
 
         xml_data = '<root xmlns:ns0="http://xmlschema.test/0">ns0:foo</root>'
-        self.assertEqual(schema.decode(xml_data), 'ns0:foo')
+        self.assertEqual(schema.decode(xml_data),
+                         {'@xmlns:ns0': 'http://xmlschema.test/0', '$': 'ns0:foo'})
 
         self.assertEqual(schema.decode('<root>foo</root>'), 'foo')
 
@@ -484,12 +694,14 @@ class TestDecoding(XsdValidatorTestCase):
         self.assertIn("Path: /root", str(ctx.exception))
 
         xml_data = '<root name="ns0:bar" xmlns:ns0="http://xmlschema.test/0">ns0:foo</root>'
-        self.assertEqual(schema.decode(xml_data), {'@name': 'ns0:bar', '$': 'ns0:foo'})
+        self.assertEqual(schema.decode(xml_data), {
+            '@xmlns:ns0': 'http://xmlschema.test/0', '@name': 'ns0:bar', '$': 'ns0:foo'
+        })
 
         # Check reverse encoding
         obj = schema.decode(xml_data, converter=JsonMLConverter)
         root = schema.encode(obj, converter=JsonMLConverter)
-        self.assertEqual(ElementTree.tostring(root), b'<root name="ns0:bar">ns0:foo</root>\n')
+        self.assertEqual(ElementTree.tostring(root), b'<root name="ns0:bar">ns0:foo</root>')
 
         with self.assertRaises(XMLSchemaValidationError) as ctx:
             schema.decode('<root name="ns0:bar">foo</root>')
@@ -596,11 +808,6 @@ class TestDecoding(XsdValidatorTestCase):
              '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
              '@xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd'})
 
-        xmlschema.limits.MAX_XML_DEPTH = 1
-        with self.assertRaises(XMLSchemaValidationError):
-            schema.decode(self.col_xml_file)
-        xmlschema.limits.MAX_XML_DEPTH = 9999
-
         self.assertEqual(
             schema.decode(self.col_xml_file, max_depth=2),
             {'@xmlns:col': 'http://example.com/ns/collection',
@@ -681,9 +888,15 @@ class TestDecoding(XsdValidatorTestCase):
         self.assertEqual(xd['decimal_value'], ['abc'])
 
     def test_datatypes(self):
+        xd = self.st_schema.to_dict(self.casepath('features/decoder/data.xml'))
+        self.assertDictEqual(xd, DATA_DICT)
+
         xt = ElementTree.parse(self.casepath('features/decoder/data.xml'))
         xd = self.st_schema.to_dict(xt, namespaces=self.default_namespaces)
-        self.assertEqual(xd, DATA_DICT)
+        self.assertNotEqual(xd, DATA_DICT)
+        self.assertIn('@xmlns:xsi', xd)
+        self.assertIn('@xmlns:tns', xd)
+        self.assertIn('@xmlns:ns', xd)
 
     def test_datetime_types(self):
         xs = self.get_schema('<xs:element name="dt" type="xs:dateTime"/>')
@@ -903,11 +1116,11 @@ class TestDecoding(XsdValidatorTestCase):
         # Element
         xs = self.get_schema('<xs:element name="b64" type="xs:base64Binary"/>')
 
-        obj = xs.decode('<b64>{}</b64>'.format(base64_value.decode()))
+        obj = xs.decode(f'<b64>{base64_value.decode()}</b64>')
         self.assertEqual(obj, str(expected_value))
         self.assertIsInstance(obj, str)
 
-        obj = xs.decode('<b64>{}</b64>'.format(base64_value.decode()), binary_types=True)
+        obj = xs.decode(f'<b64>{base64_value.decode()}</b64>', binary_types=True)
         self.assertEqual(obj, expected_value)
         self.assertIsInstance(obj, datatypes.Base64Binary)
 
@@ -915,13 +1128,13 @@ class TestDecoding(XsdValidatorTestCase):
                           datatypes.Base64Binary('YWJjZWZnaA=='))
         self.check_decode(base64_code_type, b' Y  W J j ZWZ\t\tn\na A= =',
                           datatypes.Base64Binary('Y W J j ZWZ n a A= ='))
-        self.check_decode(base64_code_type, u' Y  W J j ZWZ\t\tn\na A= =',
+        self.check_decode(base64_code_type, ' Y  W J j ZWZ\t\tn\na A= =',
                           datatypes.Base64Binary('Y W J j ZWZ n a A= ='))
         self.check_decode(base64_code_type, base64.b64encode(b'abcefghi'),
                           datatypes.Base64Binary('YWJjZWZnaGk='))
 
-        self.check_decode(base64_code_type, u'YWJjZWZnaA=', XMLSchemaValidationError)
-        self.check_decode(base64_code_type, u'YWJjZWZna$==', XMLSchemaValidationError)
+        self.check_decode(base64_code_type, 'YWJjZWZnaA=', XMLSchemaValidationError)
+        self.check_decode(base64_code_type, 'YWJjZWZna$==', XMLSchemaValidationError)
 
         base64_length4_type = self.st_schema.types['base64Length4']
         self.check_decode(base64_length4_type, base64.b64encode(b'abc'),
@@ -1037,7 +1250,8 @@ class TestDecoding(XsdValidatorTestCase):
                 element_data.tag,
                 element_data.text,
                 element_data.content,
-                [x for x in element_data.attributes if x[0] != XSI_NIL]
+                [x for x in element_data.attributes if x[0] != XSI_NIL],
+                None
             )
 
         data = schema.decode(xml_file, element_hook=element_hook)
@@ -1062,7 +1276,9 @@ class TestDecoding(XsdValidatorTestCase):
           <xs:element name="foo" type="xs:string" />
         </xs:schema>""")
         self.assertEqual(xs.to_dict("""<foo xmlns="http://example.com/foo">bar</foo>""",
-                                    path='/foo', namespaces={'': 'http://example.com/foo'}), 'bar')
+                                    path='/foo', namespaces={'': 'http://example.com/foo'}),
+                         {'@xmlns': 'http://example.com/foo', '$': 'bar'})
+
         self.assertEqual(xs.to_dict("""<foo>bar</foo>""",
                                     path='/foo', namespaces={'': 'http://example.com/foo'}), None)
 
@@ -1077,7 +1293,7 @@ class TestDecoding(XsdValidatorTestCase):
     def test_union_types__issue_103(self):
         decimal_or_nan = self.st_schema.types['myType']
         self.check_decode(decimal_or_nan, '95.0', Decimal('95.0'))
-        self.check_decode(decimal_or_nan, 'NaN', u'NaN')
+        self.check_decode(decimal_or_nan, 'NaN', 'NaN')
 
     def test_default_values__issue_108(self):
         # From issue #108
@@ -1151,22 +1367,22 @@ class TestDecoding(XsdValidatorTestCase):
         self.assertFalse(schema.is_valid(self.casepath('issues/issue_204/issue_204_2.xml')))
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_2.xml'), validation='lax')
-        self.assertEqual(set(x for x in data[0] if x[0] != '@'), {'child2', 'child5'})
+        self.assertEqual({x for x in data[0] if x[0] != '@'}, {'child2', 'child5'})
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_3.xml'), validation='lax')
-        self.assertEqual(set(x for x in data[0] if x[0] != '@'), {'child2', 'child5'})
+        self.assertEqual({x for x in data[0] if x[0] != '@'}, {'child2', 'child5'})
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_3.xml'),
                              validation='lax', keep_unknown=True)
-        self.assertEqual(set(x for x in data[0] if x[0] != '@'), {'child2', 'unknown', 'child5'})
+        self.assertEqual({x for x in data[0] if x[0] != '@'}, {'child2', 'unknown', 'child5'})
         self.assertEqual(data[0]['unknown'], {'a': ['1'], 'b': [None]})
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_2.xml'), validation='skip')
-        self.assertEqual(set(x for x in data if x[0] != '@'), {'child2', 'child5'})
+        self.assertEqual({x for x in data if x[0] != '@'}, {'child2', 'child5'})
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_3.xml'),
                              validation='skip', keep_unknown=True)
-        self.assertEqual(set(x for x in data if x[0] != '@'), {'child2', 'unknown', 'child5'})
+        self.assertEqual({x for x in data if x[0] != '@'}, {'child2', 'unknown', 'child5'})
         self.assertEqual(data['unknown'], {'a': ['1'], 'b': [None]})
 
     def test_error_message__issue_115(self):
@@ -1255,7 +1471,8 @@ class TestDecoding(XsdValidatorTestCase):
         with self.assertRaises(XMLSchemaValidationError) as ctx:
             schema.to_dict(self.casepath('issues/issue_200/issue_200.xml'),
                            path='/na:main/na:item[@doc_id=2]'),
-        self.assertIn('is not an element of the schema', str(ctx.exception))
+        self.assertIn("doesn't select any '{ames}item' element of the schema",
+                      str(ctx.exception))
 
     def test_issue_238__decode_bytes_strings(self):
         with open(self.vh_xml_file, 'rb') as fp:
@@ -1430,7 +1647,8 @@ class TestDecoding(XsdValidatorTestCase):
                 element_data.tag,
                 element_data.text,
                 filled_content,
-                element_data.attributes
+                element_data.attributes,
+                None
             )
 
         expected = {'TEST_EL': [
@@ -1463,6 +1681,140 @@ class TestDecoding(XsdValidatorTestCase):
         xml_dict = schema.decode(xml_file, element_hook=fill_missing_content, fill_missing=True)
         self.assertDictEqual(xml_dict, expected)
 
+    def test_decoding_non_unicode_files(self):
+        # Using cp1252 encoded XSD
+        schema_file = self.casepath('examples/menù/menù-cp1252.xsd')
+        schema = self.schema_class(schema_file)
+
+        xml_file = self.casepath('examples/menù/menù-cp1252.xml')
+        self.assertDictEqual(schema.decode(xml_file), MENU_DICT)
+
+        xml_file = self.casepath('examples/menù/menù-ascii.xml')
+        with self.assertRaises(ElementTree.ParseError):
+            schema.decode(xml_file)  # Invalid XML file (entity in a tag name)
+
+        xml_file = self.casepath('examples/menù/menù.xml')
+        self.assertDictEqual(schema.decode(xml_file), MENU_DICT)
+
+        # Using ASCII encoded XSD
+        schema_file = self.casepath('examples/menù/menù-ascii.xsd')
+        schema = self.schema_class(schema_file)
+
+        xml_file = self.casepath('examples/menù/menù-cp1252.xml')
+        self.assertDictEqual(schema.decode(xml_file), MENU_DICT)
+
+        xml_file = self.casepath('examples/menù/menù.xml')
+        self.assertDictEqual(schema.decode(xml_file), MENU_DICT)
+
+    def test_xml_to_json_serialization(self):
+        xml_file = self.casepath('serialization/document.xml')
+
+        obj = xmlschema.to_json(
+            xml_file,
+            validation='skip',
+            converter=xmlschema.ParkerConverter,
+            xmlns_processing='stacked',
+        )
+        with open(self.casepath('serialization/parker.json')) as fp:
+            json_data = json.load(fp)
+
+        self.assertDictEqual(json.loads(obj), json_data)
+
+        obj = xmlschema.to_json(
+            xml_file,
+            validation='skip',
+            converter=xmlschema.BadgerFishConverter,
+            xmlns_processing='stacked',
+        )
+        with open(self.casepath('serialization/badgerfish.json')) as fp:
+            json_data = json.load(fp)
+
+        self.assertDictEqual(json.loads(obj), json_data)
+
+        obj = xmlschema.to_json(
+            xml_file,
+            validation='skip',
+            converter=xmlschema.JsonMLConverter,
+            xmlns_processing='stacked',
+        )
+        with open(self.casepath('serialization/jsonml.json')) as fp:
+            json_data = json.load(fp)
+
+        self.assertListEqual(json.loads(obj), json_data)
+
+        obj = xmlschema.to_json(
+            xml_file,
+            validation='skip',
+            converter=xmlschema.AbderaConverter,
+            xmlns_processing='stacked',
+        )
+        with open(self.casepath('serialization/abdera.json')) as fp:
+            json_data = json.load(fp)
+
+        self.assertDictEqual(json.loads(obj), json_data)
+
+    def test_xmlns_processing_argument(self):
+        xsd_file = self.casepath('examples/collection/collection5.xsd')
+        xml_file = self.casepath('examples/collection/collection-redef-xmlns.xml')
+
+        schema = self.schema_class(xsd_file)
+        self.assertTrue(schema.is_valid(xml_file))
+
+        obj = schema.decode(xml_file)
+        self.assertDictEqual(obj, COLLECTION_XMLNS_PROCESSING_STACKED)
+
+        obj = schema.decode(xml_file, xmlns_processing='stacked')
+        self.assertDictEqual(obj, COLLECTION_XMLNS_PROCESSING_STACKED)
+
+        obj = schema.decode(xml_file, xmlns_processing='collapsed')
+        self.assertDictEqual(obj, COLLECTION_XMLNS_PROCESSING_COLLAPSED)
+
+        obj = schema.decode(xml_file, xmlns_processing='root-only')
+        self.assertDictEqual(obj, COLLECTION_XMLNS_PROCESSING_ROOT_ONLY)
+
+        namespaces = {'xsi-alt': 'http://www.w3.org/2001/XMLSchema-instance',
+                      'col-alt': 'http://example.com/ns/collection'}
+        obj = schema.decode(xml_file, xmlns_processing='none', namespaces=namespaces)
+        self.assertDictEqual(obj, COLLECTION_XMLNS_PROCESSING_NONE)
+
+        with self.assertRaises(ValueError):
+            schema.decode(xml_file, xmlns_processing='precise')
+
+        with self.assertRaises(TypeError):
+            schema.decode(xml_file, xmlns_processing=True)
+
+    def test_namespaces_argument(self):
+        namespaces = {'vh': 'http://example.com/vehicles'}
+        obj = self.vh_schema.decode(self.vh_xml_file, namespaces=namespaces)
+        self.assertDictEqual(obj, VEHICLES_DICT)
+
+        namespaces = {'vh2': 'http://example.com/vehicles'}
+        obj = self.vh_schema.decode(self.vh_xml_file, namespaces=namespaces)
+        self.assertDictEqual(obj, VEHICLES_DICT_OVERRIDE_PREFIX_1)
+
+        namespaces = {'vh-alt': 'http://example.com/vehicles',
+                      '': 'http://example.com/vehicles'}
+        obj = self.vh_schema.decode(self.vh_xml_file, namespaces=namespaces)
+        self.assertDictEqual(obj, VEHICLES_DICT_OVERRIDE_PREFIX_2)
+
+        namespaces = {'': 'http://example.com/vehicles'}
+        obj = self.vh_schema.decode(self.vh_xml_file, namespaces=namespaces)
+        self.assertDictEqual(obj, VEHICLES_DICT_OVERRIDE_PREFIX_3)
+
+    def test_validation_hook_argument(self):
+        tags_num = 1
+
+        def stop_decoding(_e, _xsd_element):
+            nonlocal tags_num
+            if tags_num >= 10:
+                return True
+            tags_num += 1
+            return False
+
+        obj = self.col_schema.decode(self.col_xml_file, validation_hook=stop_decoding)
+        self.assertIn('object', obj)
+        self.assertEqual(len(obj['object']), 1)
+
 
 class TestDecoding11(TestDecoding):
     schema_class = XMLSchema11
@@ -1516,9 +1868,5 @@ class TestDecoding11(TestDecoding):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema decoding with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('decoding')
