@@ -20,11 +20,15 @@ from xmlschema.names import XSD_NAMESPACE, XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_L
     XSD_WHITE_SPACE, XSD_MIN_INCLUSIVE, XSD_MIN_EXCLUSIVE, XSD_MAX_INCLUSIVE, \
     XSD_MAX_EXCLUSIVE, XSD_TOTAL_DIGITS, XSD_FRACTION_DIGITS, XSD_ENUMERATION, \
     XSD_PATTERN, XSD_ASSERTION
+from xmlschema.validators import XsdEnumerationFacets, XsdPatternFacets, \
+    XsdAssertionFacet, XsdBuilders
 
 
 class TestXsdFacets(unittest.TestCase):
 
     schema_class = XMLSchema10
+    st_xsd_file: pathlib.Path
+    st_schema: XMLSchema10
 
     @classmethod
     def setUpClass(cls):
@@ -237,7 +241,7 @@ class TestXsdFacets(unittest.TestCase):
                     <xs:restriction base="string20">
                         <xs:minLength value="30"/>
                     </xs:restriction>
-                </xs:simpleType>                
+                </xs:simpleType>
             </xs:schema>"""))
 
         self.assertEqual(schema.types['string20'].get_facet(XSD_MIN_LENGTH).value, 20)
@@ -255,7 +259,7 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="string40">
                             <xs:minLength value="30"/>
                         </xs:restriction>
-                    </xs:simpleType>                
+                    </xs:simpleType>
                 </xs:schema>"""))
 
     def test_max_length_facet(self):
@@ -298,7 +302,7 @@ class TestXsdFacets(unittest.TestCase):
                     <xs:restriction base="string30">
                         <xs:maxLength value="20"/>
                     </xs:restriction>
-                </xs:simpleType>                
+                </xs:simpleType>
             </xs:schema>"""))
 
         self.assertEqual(schema.types['string30'].get_facet(XSD_MAX_LENGTH).value, 30)
@@ -316,7 +320,7 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="string30">
                             <xs:maxLength value="40"/>
                         </xs:restriction>
-                    </xs:simpleType>                
+                    </xs:simpleType>
                 </xs:schema>"""))
 
     def test_min_inclusive_facet(self):
@@ -363,10 +367,10 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="type1">
                             <xs:minInclusive value="0"/>
                         </xs:restriction>
-                    </xs:simpleType>                    
+                    </xs:simpleType>
                 </xs:schema>"""))
 
-            facet = schema.types['type1'].get_facet('{%s}%s' % (XSD_NAMESPACE, base_facet))
+            facet = schema.types['type1'].get_facet(f'{{{XSD_NAMESPACE}}}{base_facet}')
             self.assertIsNone(facet(0))
             facet2 = schema.types['type2'].get_facet(XSD_MIN_INCLUSIVE)
             self.assertIsNone(facet2(0))
@@ -385,7 +389,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['maxInclusive', 'maxExclusive']:
@@ -401,7 +405,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['minExclusive', 'maxExclusive']:
@@ -417,7 +421,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
     def test_min_exclusive_facet(self):
@@ -463,10 +467,10 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="type1">
                             <xs:minExclusive value="0"/>
                         </xs:restriction>
-                    </xs:simpleType>                    
+                    </xs:simpleType>
                 </xs:schema>"""))
 
-            facet = schema.types['type1'].get_facet('{%s}%s' % (XSD_NAMESPACE, base_facet))
+            facet = schema.types['type1'].get_facet(f'{{{XSD_NAMESPACE}}}{base_facet}')
             self.assertIsNone(facet(1))
             facet2 = schema.types['type2'].get_facet(XSD_MIN_EXCLUSIVE)
             self.assertIsNone(facet2(1))
@@ -485,7 +489,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['minInclusive', 'minExclusive']:
@@ -501,7 +505,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['maxInclusive', 'maxExclusive']:
@@ -517,7 +521,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:minExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
     def test_max_inclusive_facet(self):
@@ -536,7 +540,7 @@ class TestXsdFacets(unittest.TestCase):
 
         with self.assertRaises(XMLSchemaValidationError) as ec:
             facet(1)
-        self.assertIn('value has to be lesser or equal than 0', str(ec.exception))
+        self.assertIn('value has to be less than or equal than 0', str(ec.exception))
 
         with self.assertRaises(XMLSchemaValidationError):
             facet('')
@@ -564,10 +568,10 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="type1">
                             <xs:maxInclusive value="0"/>
                         </xs:restriction>
-                    </xs:simpleType>                    
+                    </xs:simpleType>
                 </xs:schema>"""))
 
-            facet = schema.types['type1'].get_facet('{%s}%s' % (XSD_NAMESPACE, base_facet))
+            facet = schema.types['type1'].get_facet(f'{{{XSD_NAMESPACE}}}{base_facet}')
             self.assertIsNone(facet(0))
             facet2 = schema.types['type2'].get_facet(XSD_MAX_INCLUSIVE)
             self.assertIsNone(facet2(0))
@@ -586,7 +590,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['minInclusive', 'minExclusive']:
@@ -602,7 +606,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['minExclusive', 'maxExclusive']:
@@ -618,7 +622,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxInclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
     def test_max_exclusive_facet(self):
@@ -664,10 +668,10 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="type1">
                             <xs:maxExclusive value="0"/>
                         </xs:restriction>
-                    </xs:simpleType>                    
+                    </xs:simpleType>
                 </xs:schema>"""))
 
-            facet = schema.types['type1'].get_facet('{%s}%s' % (XSD_NAMESPACE, base_facet))
+            facet = schema.types['type1'].get_facet(f'{{{XSD_NAMESPACE}}}{base_facet}')
             self.assertIsNone(facet(-1))
             facet2 = schema.types['type2'].get_facet(XSD_MAX_EXCLUSIVE)
             self.assertIsNone(facet2(-1))
@@ -686,7 +690,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['maxInclusive', 'maxExclusive']:
@@ -702,7 +706,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
         for base_facet in ['minInclusive', 'minExclusive']:
@@ -718,7 +722,7 @@ class TestXsdFacets(unittest.TestCase):
                             <xs:restriction base="type1">
                                 <xs:maxExclusive value="0"/>
                             </xs:restriction>
-                        </xs:simpleType>                    
+                        </xs:simpleType>
                     </xs:schema>"""))
 
     def test_total_digits_facet(self):
@@ -823,7 +827,7 @@ class TestXsdFacets(unittest.TestCase):
                     </xs:simpleType>
                 </xs:schema>"""))
 
-        self.assertIn("value has to be 0 for types derived from xs:integer", str(ec.exception))
+        self.assertIn("value must be 0 for types derived from xs:integer", str(ec.exception))
 
         with self.assertRaises(XMLSchemaParseError) as ec:
             self.schema_class(dedent("""\
@@ -902,6 +906,8 @@ class TestXsdFacets(unittest.TestCase):
         self.assertFalse(schema.types['enum2'].is_valid('four'))
 
         facet = schema.types['enum2'].get_facet(XSD_ENUMERATION)
+        self.assertIsInstance(facet, XsdEnumerationFacets)
+
         elem = ElementTree.Element(XSD_ENUMERATION, value='three')
         facet.append(elem)
         self.assertTrue(schema.types['enum2'].is_valid('three'))
@@ -1029,6 +1035,7 @@ class TestXsdFacets(unittest.TestCase):
             </xs:schema>"""))
 
         facet = schema.types['pattern1'].get_facet(XSD_PATTERN)
+        self.assertIsInstance(facet, XsdPatternFacets)
         self.assertIsNone(facet('abc'))
         self.assertRaises(XMLSchemaValidationError, facet, '')
         self.assertRaises(XMLSchemaValidationError, facet, 'a;')
@@ -1060,13 +1067,65 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="xs:string">
                             <xs:pattern value="]"/>
                         </xs:restriction>
-                    </xs:simpleType>                    
+                    </xs:simpleType>
                 </xs:schema>"""), validation='lax')
 
         self.assertEqual(len(schema.all_errors), 2)
         self.assertIn("missing required attribute 'value'", str(schema.all_errors[0]))
         self.assertIn("unexpected meta character ']' at position 0", str(schema.all_errors[1]))
 
+    def test_get_annotation__issue_255(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:simpleType name="enum1">
+                    <xs:restriction base="xs:string">
+                        <xs:enumeration value="one">
+                            <xs:annotation>
+                                <xs:documentation>1st facet</xs:documentation>
+                            </xs:annotation>
+                        </xs:enumeration>
+                        <xs:enumeration value="two"/>
+                        <xs:enumeration value="three">
+                            <xs:annotation>
+                                <xs:documentation>3rd facet</xs:documentation>
+                            </xs:annotation>
+                        </xs:enumeration>
+                    </xs:restriction>
+                </xs:simpleType>
+            </xs:schema>"""))
+
+        facet = schema.types['enum1'].get_facet(XSD_ENUMERATION)
+        self.assertIsInstance(facet, XsdEnumerationFacets)
+        self.assertEqual(facet.annotation.documentation[0].text, '1st facet')
+        self.assertEqual(facet.get_annotation(0).documentation[0].text, '1st facet')
+        self.assertIsNone(facet.get_annotation(1))
+        self.assertEqual(facet.get_annotation(2).documentation[0].text, '3rd facet')
+
+        with self.assertRaises(IndexError):
+            facet.get_annotation(3)
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:simpleType name="pattern1">
+                    <xs:restriction base="xs:string">
+                        <xs:pattern value="\\w+"/>
+                        <xs:pattern value=".+">
+                            <xs:annotation>
+                                <xs:documentation>2nd facet</xs:documentation>
+                            </xs:annotation>
+                        </xs:pattern>
+                    </xs:restriction>
+                </xs:simpleType>
+            </xs:schema>"""))
+
+        facet = schema.types['pattern1'].get_facet(XSD_PATTERN)
+        self.assertIsInstance(facet, XsdPatternFacets)
+        self.assertIsNone(facet.get_annotation(0))
+        self.assertEqual(facet.get_annotation(1).documentation[0].text, '2nd facet')
+
+        with self.assertRaises(IndexError):
+            facet.get_annotation(2)
+
     def test_fixed_value(self):
         with self.assertRaises(XMLSchemaParseError) as ec:
             self.schema_class(dedent("""\
@@ -1080,11 +1139,85 @@ class TestXsdFacets(unittest.TestCase):
                         <xs:restriction base="string30">
                             <xs:maxLength value="20"/>
                         </xs:restriction>
-                    </xs:simpleType>                
+                    </xs:simpleType>
                 </xs:schema>"""))
 
         self.assertIn("'maxLength' facet value is fixed to 30", str(ec.exception))
 
+    def test_restriction_on_list__issue_396(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+              <xs:element name="list_of_strings">
+                <xs:simpleType>
+                  <xs:restriction>
+                    <xs:simpleType>
+                      <xs:list>
+                        <xs:simpleType>
+                          <xs:restriction base="xs:string">
+                            <xs:minLength value="5"/>
+                            <xs:maxLength value="6"/>
+                          </xs:restriction>
+                        </xs:simpleType>
+                      </xs:list>
+                    </xs:simpleType>
+                    <xs:minLength value="1"/>
+                    <xs:maxLength value="6"/>
+                  </xs:restriction>
+                </xs:simpleType>
+              </xs:element>
+            </xs:schema>"""))
+
+        self.assertTrue(schema.is_valid('<list_of_strings>abcde</list_of_strings>'))
+        self.assertTrue(schema.is_valid('<list_of_strings>abcdef</list_of_strings>'))
+        self.assertFalse(schema.is_valid('<list_of_strings>abcd</list_of_strings>'))
+        self.assertFalse(schema.is_valid('<list_of_strings>abcdefg</list_of_strings>'))
+        self.assertFalse(schema.is_valid('<list_of_strings>     </list_of_strings>'))
+
+        self.assertTrue(schema.is_valid('<list_of_strings>abcde abcde abcde '
+                                        'abcde abcde abcde</list_of_strings>'))
+        self.assertFalse(schema.is_valid('<list_of_strings>abcde abcde abcde '
+                                         'abcde abcd abcde</list_of_strings>'))
+        self.assertFalse(schema.is_valid('<list_of_strings>abcde abcde abcde '
+                                         'abcde abcde abcde abcde</list_of_strings>'))
+
+    def test_custom_pattern_facets__issue_440(self):
+
+        class UnanchoredXsdPatternFacets(XsdPatternFacets):
+            anchors = True
+
+        class MyXMLSchema(self.schema_class):
+            builders = XsdBuilders(None, UnanchoredXsdPatternFacets)
+
+        schema = MyXMLSchema(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:simpleType name="pattern1">
+                    <xs:restriction base="xs:string">
+                        <xs:pattern value="^\\w+$"/>
+                    </xs:restriction>
+                </xs:simpleType>
+            </xs:schema>"""))
+
+        facet = schema.types['pattern1'].get_facet(XSD_PATTERN)
+        self.assertIsInstance(facet, UnanchoredXsdPatternFacets)
+        self.assertIsNone(facet('abc'))
+        self.assertRaises(XMLSchemaValidationError, facet, '')
+        self.assertRaises(XMLSchemaValidationError, facet, 'a;')
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:simpleType name="pattern1">
+                    <xs:restriction base="xs:string">
+                        <xs:pattern value="^\\w+$"/>
+                    </xs:restriction>
+                </xs:simpleType>
+            </xs:schema>"""))
+
+        facet = schema.types['pattern1'].get_facet(XSD_PATTERN)
+        self.assertIsInstance(facet, XsdPatternFacets)
+        self.assertRaises(XMLSchemaValidationError, facet, 'abc')
+        self.assertRaises(XMLSchemaValidationError, facet, '')
+        self.assertRaises(XMLSchemaValidationError, facet, 'a;')
+
 
 class TestXsd11Identities(TestXsdFacets):
 
@@ -1192,7 +1325,7 @@ class TestXsd11Identities(TestXsdFacets):
                 </xs:simpleType>
                 <xs:simpleType name="string2">
                     <xs:restriction base="xs:string">
-                        <xs:assertion test="last()" 
+                        <xs:assertion test="last()"
                                       xpathDefaultNamespace="http://xpath.test/ns"/>
                     </xs:restriction>
                 </xs:simpleType>
@@ -1201,7 +1334,6 @@ class TestXsd11Identities(TestXsdFacets):
                         <xs:assertion test="position()"/>
                     </xs:restriction>
                 </xs:simpleType>
-                
                 <xs:simpleType name="integer_list">
                     <xs:list itemType="xs:integer"/>
                 </xs:simpleType>
@@ -1213,10 +1345,12 @@ class TestXsd11Identities(TestXsdFacets):
             </xs:schema>"""))
 
         facet = schema.types['string1'].get_facet(XSD_ASSERTION)
+        self.assertIsInstance(facet, XsdAssertionFacet)
         self.assertIsNone(facet(''))
         self.assertEqual(facet.xpath_default_namespace, '')
 
         facet = schema.types['string2'].get_facet(XSD_ASSERTION)
+        self.assertIsInstance(facet, XsdAssertionFacet)
         self.assertEqual(facet.xpath_default_namespace, 'http://xpath.test/ns')
         with self.assertRaises(XMLSchemaValidationError) as ec:
             facet('')
@@ -1228,6 +1362,7 @@ class TestXsd11Identities(TestXsdFacets):
 
         facet = schema.types['integer_vector'].get_facet(XSD_ASSERTION)
         self.assertIsNone(facet([1, 2, 3]))
+        self.assertIsInstance(facet, XsdAssertionFacet)
         self.assertEqual(facet.parser.variable_types, {'value': 'xs:anySimpleType'})
 
         schema = self.schema_class(dedent("""\
@@ -1248,11 +1383,33 @@ class TestXsd11Identities(TestXsdFacets):
         self.assertIn("missing attribute 'test'", str(schema.all_errors[0]))
         self.assertIn("[err:XPST0003] unexpected '?' symbol", str(schema.all_errors[1]))
 
+    def test_use_xpath3(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root" type="rootType"/>
+                <xs:simpleType name="rootType">
+                    <xs:restriction base="xs:string">
+                        <xs:assertion test="let $foo := 'bar' return $foo"/>
+                    </xs:restriction>
+                </xs:simpleType>
+            </xs:schema>"""), use_xpath3=True)
+
+        self.assertTrue(schema.use_xpath3)
+
+        with self.assertRaises(XMLSchemaParseError) as ctx:
+            self.schema_class(dedent("""\
+                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                    <xs:element name="root" type="rootType"/>
+                    <xs:simpleType name="rootType">
+                        <xs:restriction base="xs:string">
+                            <xs:assertion test="let $foo := 'bar' return $foo"/>
+                        </xs:restriction>
+                    </xs:simpleType>
+                </xs:schema>"""))
+
+        self.assertIn('XPST0003', str(ctx.exception))
+
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema's XSD facets with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('XSD facets')
